"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.prepareZkpProof = exports.packZkpProof = exports.prepareAuthV2ZeroKnowledgeResponse = exports.calculateRequestIdForCircuit = void 0;
const ethers_1 = require("ethers");
const utils_1 = require("../../utils");
const circuits_1 = require("../../circuits");
const js_crypto_1 = require("@iden3/js-crypto");
/**
 * @beta
 * Retrieves the request ID from circuit string.
 * CircuitId.AuthV2 - 940499666
 * @returns The request ID.
 */
function calculateRequestIdForCircuit(circuitId) {
    const circuitHash = (0, ethers_1.keccak256)(utils_1.byteEncoder.encode(circuitId));
    const dataView = new DataView(js_crypto_1.Hex.decodeString(circuitHash.replace('0x', '')).buffer);
    const id = dataView.getUint32(0);
    return id;
}
exports.calculateRequestIdForCircuit = calculateRequestIdForCircuit;
/**
 * Prepares the zero-knowledge proof response for the AuthV2 circuit.
 * @beta
 * @param address - The address associated with the request.
 * @param senderDid - The sender's decentralized identifier (DID).
 * @param proofService - The proof service used to generate the proof.
 * @returns A promise that resolves to an array of ZeroKnowledgeProofResponse objects.
 */
async function prepareAuthV2ZeroKnowledgeResponse(address, senderDid, proofService) {
    const circuitId = circuits_1.CircuitId.AuthV2;
    //  this is now hardcoded calculated value for 'authV2' that can be changed in the future.
    const id = 940499666;
    if ((0, utils_1.isEthereumIdentity)(senderDid)) {
        return [
            {
                circuitId,
                id,
                pub_signals: [],
                proof: {
                    pi_a: [],
                    pi_b: [],
                    pi_c: [],
                    protocol: 'groth16'
                }
            }
        ];
    }
    const hash = Uint8Array.from([...(0, utils_1.hexToBytes)(address), ...new Uint8Array(12)]).reverse();
    const { proof, pub_signals } = await proofService.generateAuthV2Proof(hash, senderDid);
    return [
        {
            circuitId,
            id,
            pub_signals,
            proof
        }
    ];
}
exports.prepareAuthV2ZeroKnowledgeResponse = prepareAuthV2ZeroKnowledgeResponse;
const packZkpProof = (inputs, a, b, c) => {
    return new ethers_1.ethers.AbiCoder().encode(['uint256[] inputs', 'uint256[2]', 'uint256[2][2]', 'uint256[2]'], [inputs, a, b, c]);
};
exports.packZkpProof = packZkpProof;
const prepareZkpProof = (proof) => {
    return {
        a: proof.pi_a.slice(0, 2),
        b: [
            [proof.pi_b[0][1], proof.pi_b[0][0]],
            [proof.pi_b[1][1], proof.pi_b[1][0]]
        ],
        c: proof.pi_c.slice(0, 2)
    };
};
exports.prepareZkpProof = prepareZkpProof;
