import { getRandomBytes } from '@iden3/js-crypto';
import { AuthMethod } from '../types';
import { bytesToHex, mergeObjects } from '../../utils';
import { BytesHelper, DID, getUnixTimestamp } from '@iden3/js-iden3-core';
import { CircuitId } from '../../circuits';
import { AcceptJwsAlgorithms, defaultAcceptProfile, MediaType } from '../constants';
import { packZkpProof, prepareZkpProof } from '../utils';
/**
 * Groups the ZeroKnowledgeProofRequest objects based on their groupId.
 * Returns a Map where the key is the groupId and the value is an object containing the query and linkNonce.
 *
 * @param requestScope - An array of ZeroKnowledgeProofRequest objects.
 * @returns A Map<number, { query: ZeroKnowledgeProofQuery; linkNonce: number }> representing the grouped queries.
 */
const getGroupedQueries = (requestScope) => requestScope.reduce((acc, proofReq) => {
    const groupId = proofReq.query.groupId;
    if (!groupId) {
        return acc;
    }
    const existedData = acc.get(groupId);
    if (!existedData) {
        const seed = getRandomBytes(12);
        const dataView = new DataView(seed.buffer);
        const linkNonce = dataView.getUint32(0);
        acc.set(groupId, { query: proofReq.query, linkNonce });
        return acc;
    }
    const credentialSubject = mergeObjects(existedData.query.credentialSubject, proofReq.query.credentialSubject);
    acc.set(groupId, {
        ...existedData,
        query: {
            skipClaimRevocationCheck: existedData.query.skipClaimRevocationCheck || proofReq.query.skipClaimRevocationCheck,
            ...existedData.query,
            credentialSubject
        }
    });
    return acc;
}, new Map());
/**
 * Processes zero knowledge proof requests.
 *
 * @param to - The identifier of the recipient.
 * @param requests - An array of zero knowledge proof requests.
 * @param from - The identifier of the sender.
 * @param proofService - The proof service.
 * @param opts - Additional options for processing the requests.
 * @returns A promise that resolves to an array of zero knowledge proof responses.
 */
export const processZeroKnowledgeProofRequests = async (to, requests, from, proofService, opts) => {
    const requestScope = requests ?? [];
    const combinedQueries = getGroupedQueries(requestScope);
    const groupedCredentialsCache = new Map();
    const zkpResponses = [];
    for (const proofReq of requestScope) {
        if (!opts.supportedCircuits.includes(proofReq.circuitId)) {
            throw new Error(`Circuit ${proofReq.circuitId} is not allowed`);
        }
        const query = proofReq.query;
        const groupId = query.groupId;
        const combinedQueryData = combinedQueries.get(groupId);
        if (groupId) {
            if (!combinedQueryData) {
                throw new Error(`Invalid group id ${query.groupId}`);
            }
            const combinedQuery = combinedQueryData.query;
            if (!groupedCredentialsCache.has(groupId)) {
                const credWithRevStatus = await proofService.findCredentialByProofQuery(to, combinedQueryData.query);
                if (!credWithRevStatus.cred) {
                    throw new Error(`Credential not found for query ${JSON.stringify(combinedQuery)}`);
                }
                groupedCredentialsCache.set(groupId, credWithRevStatus);
            }
        }
        const credWithRevStatus = groupedCredentialsCache.get(groupId);
        const zkpRes = await proofService.generateProof(proofReq, to, {
            verifierDid: from,
            challenge: opts.challenge,
            skipRevocation: Boolean(query.skipClaimRevocationCheck),
            credential: credWithRevStatus?.cred,
            credentialRevocationStatus: credWithRevStatus?.revStatus,
            linkNonce: combinedQueryData?.linkNonce ? BigInt(combinedQueryData.linkNonce) : undefined
        });
        zkpResponses.push(zkpRes);
    }
    return zkpResponses;
};
/**
 * Processes auth proof requests.
 *
 * @param to - The identifier of the recipient.
 * @param proofService - The proof service.
 * @param opts - Additional options for processing the requests.
 * @returns A promise that resolves to an auth proof response.
 */
export const processProofAuth = async (to, proofService, opts) => {
    if (!opts.acceptProfile) {
        opts.acceptProfile = defaultAcceptProfile;
    }
    if (!opts.skipRevocation) {
        opts.skipRevocation = true;
    }
    switch (opts.acceptProfile.env) {
        case MediaType.ZKPMessage:
            if (!opts.acceptProfile.circuits) {
                throw new Error('Circuit not specified in accept profile');
            }
            for (const circuitId of opts.acceptProfile.circuits) {
                if (!opts.supportedCircuits.includes(circuitId)) {
                    throw new Error(`Circuit ${circuitId} is not supported`);
                }
                const zkpRes = await proofService.generateAuthProof(circuitId, to, { challenge: opts.challenge, skipRevocation: opts.skipRevocation });
                switch (circuitId) {
                    case CircuitId.AuthV2: {
                        const preparedZkpProof = prepareZkpProof(zkpRes.proof);
                        const zkProofEncoded = packZkpProof(zkpRes.pub_signals, preparedZkpProof.a, preparedZkpProof.b, preparedZkpProof.c);
                        return {
                            authMethod: AuthMethod.AUTHV2,
                            proof: zkProofEncoded
                        };
                    }
                }
            }
            throw new Error(`Auth method is not supported`);
        case MediaType.SignedMessage:
            if (!opts.acceptProfile.alg || opts.acceptProfile.alg.length === 0) {
                throw new Error('Algorithm not specified');
            }
            if (opts.acceptProfile.alg[0] === AcceptJwsAlgorithms.ES256KR) {
                const ethIdProof = packEthIdentityProof(to);
                return {
                    authMethod: AuthMethod.ETH_IDENTITY,
                    proof: ethIdProof
                };
            }
            throw new Error(`Algorithm ${opts.acceptProfile.alg[0]} not supported`);
        default:
            throw new Error('Accept env not supported');
    }
};
/**
 * Verifies that the expires_time field of a message is not in the past. Throws an error if it is.
 *
 * @param message - Basic message to verify.
 */
export const verifyExpiresTime = (message) => {
    if (message?.expires_time && message.expires_time < getUnixTimestamp(new Date())) {
        throw new Error('Message expired');
    }
};
export const packEthIdentityProof = (did) => {
    return `0x${bytesToHex(BytesHelper.intToBytes(DID.idFromDID(did).bigInt()))}`;
};
