import { JsonRpcProvider, Contract, ethers } from 'ethers';
import abi from './abi/ZkpVerifier.json';
//import { UniversalVerifier } from '@iden3/universal-verifier-v2-abi';
import { IVerifierABI } from '@iden3/universal-verifier-v2-abi';
import { TransactionService } from '../../blockchain';
import { chainIDfromDID, DID, Id } from '@iden3/js-iden3-core';
import { AtomicQueryMTPV2OnChainPubSignals, AtomicQuerySigV2OnChainPubSignals, AtomicQueryV3OnChainPubSignals, AuthV2PubSignals, CircuitId } from '../../circuits';
import { byteEncoder, DIDDocumentSignature, resolveDidDocument } from '../../utils';
import { poseidon } from '@iden3/js-crypto';
import { Hash } from '@iden3/js-merkletree';
import { packZkpProof, prepareZkpProof } from '../../iden3comm/utils';
const maxGasLimit = 10000000n;
/**
 * Supported function signature for SubmitZKPResponse
 */
export var FunctionSignatures;
(function (FunctionSignatures) {
    /**
     * solidity identifier for function signature:
     * function submitZKPResponse(uint64 requestId, uint256[] calldata inputs,
     * uint256[2] calldata a, uint256[2][2] calldata b, uint256[2] calldata c) public
     */
    FunctionSignatures["SubmitZKPResponseV1"] = "b68967e2";
    //function submitZKPResponseV2(tuple[](uint64 requestId,bytes zkProof,bytes data),bytes crossChainProof)
    FunctionSignatures["SubmitZKPResponseV2"] = "ade09fcd";
    //function submitResponse(tuple(string authMethod,bytes proof),tuple(uint256 requestId,bytes proof,bytes metadata)[],bytes crossChainProof)
    FunctionSignatures["SubmitResponse"] = "06c86a91";
})(FunctionSignatures || (FunctionSignatures = {}));
/**
 * OnChainZKPVerifier is a class that allows to interact with the OnChainZKPVerifier contract
 * and submitZKPResponse.
 *
 * @beta
 * @class OnChainZKPVerifier
 */
export class OnChainZKPVerifier {
    /**
     *
     * Creates an instance of OnChainZKPVerifier.
     * @beta
     * @param {EthConnectionConfig[]} - array of ETH configs
     */
    constructor(_configs, _opts) {
        this._configs = _configs;
        this._opts = _opts;
    }
    static async prepareTxArgsSubmitV1(txData, zkProofResponse) {
        if (txData.method_id.replace('0x', '') !== FunctionSignatures.SubmitZKPResponseV1) {
            throw new Error(`prepareTxArgsSubmitV1 function doesn't implement requested method id. Only '0x${FunctionSignatures.SubmitZKPResponseV1}' is supported.`);
        }
        const requestID = zkProofResponse.id;
        const inputs = zkProofResponse.pub_signals;
        const preparedZkpProof = prepareZkpProof(zkProofResponse.proof);
        const payload = [requestID, inputs, preparedZkpProof.a, preparedZkpProof.b, preparedZkpProof.c];
        return payload;
    }
    /**
     * {@inheritDoc IOnChainZKPVerifier.prepareTxArgsSubmitV1}
     */
    async prepareTxArgsSubmitV1(txData, zkProofResponse) {
        return OnChainZKPVerifier.prepareTxArgsSubmitV1(txData, zkProofResponse);
    }
    /**
     * {@inheritDoc IOnChainZKPVerifier.submitZKPResponse}
     */
    async submitZKPResponse(ethSigner, txData, zkProofResponses) {
        const chainConfig = this._configs.find((i) => i.chainId == txData.chain_id);
        if (!chainConfig) {
            throw new Error(`config for chain id ${txData.chain_id} was not found`);
        }
        if (txData.method_id.replace('0x', '') !== FunctionSignatures.SubmitZKPResponseV1) {
            throw new Error(`submitZKPResponse function doesn't implement requested method id. Only '0x${FunctionSignatures.SubmitZKPResponseV1}' is supported.`);
        }
        const provider = new JsonRpcProvider(chainConfig.url, chainConfig.chainId);
        ethSigner = ethSigner.connect(provider);
        const response = new Map();
        const feeData = await provider.getFeeData();
        const maxFeePerGas = chainConfig.maxFeePerGas
            ? BigInt(chainConfig.maxFeePerGas)
            : feeData.maxFeePerGas;
        const maxPriorityFeePerGas = chainConfig.maxPriorityFeePerGas
            ? BigInt(chainConfig.maxPriorityFeePerGas)
            : feeData.maxPriorityFeePerGas;
        const verifierContract = new Contract(txData.contract_address, abi);
        for (const zkProofResponse of zkProofResponses) {
            const txArgs = await this.prepareTxArgsSubmitV1(txData, zkProofResponse);
            const payload = await verifierContract.submitZKPResponse.populateTransaction(...txArgs);
            const request = {
                to: txData.contract_address,
                data: payload.data,
                maxFeePerGas,
                maxPriorityFeePerGas
            };
            let gasLimit;
            try {
                gasLimit = await ethSigner.estimateGas(request);
            }
            catch (e) {
                gasLimit = maxGasLimit;
            }
            request.gasLimit = gasLimit;
            const transactionService = new TransactionService(provider);
            const { txnHash } = await transactionService.sendTransactionRequest(ethSigner, request);
            response.set(txnHash, zkProofResponse);
        }
        return response;
    }
    /**
     * {@inheritDoc IOnChainZKPVerifier.submitZKPResponseV2}
     */
    async submitZKPResponseV2(ethSigner, txData, zkProofResponses) {
        const chainConfig = this._configs.find((i) => i.chainId == txData.chain_id);
        if (!chainConfig) {
            throw new Error(`config for chain id ${txData.chain_id} was not found`);
        }
        if (txData.method_id.replace('0x', '') !== FunctionSignatures.SubmitZKPResponseV2) {
            throw new Error(`submitZKPResponseV2 function doesn't implement requested method id. Only '0x${FunctionSignatures.SubmitZKPResponseV2}' is supported.`);
        }
        if (!this._opts?.didResolverUrl) {
            throw new Error(`did resolver url required for crosschain verification`);
        }
        const provider = new JsonRpcProvider(chainConfig.url, chainConfig.chainId);
        ethSigner = ethSigner.connect(provider);
        const txDataArgs = await this.prepareTxArgsSubmitV2(txData, zkProofResponses);
        const feeData = await provider.getFeeData();
        const maxFeePerGas = chainConfig.maxFeePerGas
            ? BigInt(chainConfig.maxFeePerGas)
            : feeData.maxFeePerGas;
        const maxPriorityFeePerGas = chainConfig.maxPriorityFeePerGas
            ? BigInt(chainConfig.maxPriorityFeePerGas)
            : feeData.maxPriorityFeePerGas;
        const verifierContract = new Contract(txData.contract_address, abi);
        const txRequestData = await verifierContract.submitZKPResponseV2.populateTransaction(...txDataArgs);
        const request = {
            to: txData.contract_address,
            data: txRequestData.data,
            maxFeePerGas,
            maxPriorityFeePerGas
        };
        let gasLimit;
        try {
            gasLimit = await ethSigner.estimateGas(request);
        }
        catch (e) {
            gasLimit = maxGasLimit;
        }
        request.gasLimit = gasLimit;
        const transactionService = new TransactionService(provider);
        const { txnHash } = await transactionService.sendTransactionRequest(ethSigner, request);
        return new Map().set(txnHash, zkProofResponses);
    }
    /**
     * {@inheritDoc IOnChainVerifierMultiQuery.submitResponse}
     */
    async submitResponse(ethSigner, txData, authResponse, responses) {
        const chainConfig = this._configs.find((i) => i.chainId == txData.chain_id);
        if (!chainConfig) {
            throw new Error(`config for chain id ${txData.chain_id} was not found`);
        }
        if (txData.method_id.replace('0x', '') !== FunctionSignatures.SubmitResponse) {
            throw new Error(`submitResponse function doesn't implement requested method id. Only '0x${FunctionSignatures.SubmitResponse}' is supported.`);
        }
        if (!this._opts?.didResolverUrl) {
            throw new Error(`did resolver url required for crosschain verification`);
        }
        const provider = new JsonRpcProvider(chainConfig.url, chainConfig.chainId);
        ethSigner = ethSigner.connect(provider);
        const txDataArgs = await this.prepareTxArgsSubmit(txData, authResponse, responses);
        const feeData = await provider.getFeeData();
        const maxFeePerGas = chainConfig.maxFeePerGas
            ? BigInt(chainConfig.maxFeePerGas)
            : feeData.maxFeePerGas;
        const maxPriorityFeePerGas = chainConfig.maxPriorityFeePerGas
            ? BigInt(chainConfig.maxPriorityFeePerGas)
            : feeData.maxPriorityFeePerGas;
        const verifierContract = new Contract(txData.contract_address, IVerifierABI);
        const txRequestData = await verifierContract.submitResponse.populateTransaction(...txDataArgs);
        const request = {
            to: txData.contract_address,
            data: txRequestData.data,
            maxFeePerGas,
            maxPriorityFeePerGas
        };
        let gasLimit;
        try {
            gasLimit = await ethSigner.estimateGas(request);
        }
        catch (e) {
            gasLimit = maxGasLimit;
        }
        request.gasLimit = gasLimit;
        const transactionService = new TransactionService(provider);
        const { txnHash } = await transactionService.sendTransactionRequest(ethSigner, request);
        // return multiple responses for all the responses (single and grouped)
        return new Map().set(txnHash, {
            responses: responses,
            authProofs: [authResponse],
            crossChainProofs: [txDataArgs[2]]
        });
    }
    static async prepareTxArgsSubmit(resolverUrl, txData, authResponse, responses) {
        if (txData.method_id.replace('0x', '') !== FunctionSignatures.SubmitResponse) {
            throw new Error(`submit cross chain doesn't implement requested method id. Only '0x${FunctionSignatures.SubmitResponse}' is supported.`);
        }
        const gistUpdateArr = [];
        const stateUpdateArr = [];
        const payloadResponses = [];
        // Process all the responses
        for (const zkProof of responses) {
            const { requestID, zkProofEncoded, metadata } = await this._processProof(zkProof);
            payloadResponses.push({
                requestId: requestID,
                proof: zkProofEncoded,
                metadata: metadata
            });
        }
        const crossChainProofs = this.packCrossChainProofs(gistUpdateArr, stateUpdateArr);
        return [authResponse, payloadResponses, crossChainProofs];
    }
    async prepareTxArgsSubmit(txData, authResponse, responses) {
        if (!this._opts?.didResolverUrl) {
            throw new Error(`did resolver url required for crosschain verification`);
        }
        return OnChainZKPVerifier.prepareTxArgsSubmit(this._opts.didResolverUrl, txData, authResponse, responses);
    }
    static getCrossChainResolvers(source, txDataChainId, type, didResolverUrl) {
        return [
            ...new Set(source.map((info) => JSON.stringify({
                id: info.id.string(),
                [type]: type === 'gist' ? info.root?.string() : info.state?.string()
            })))
        ].reduce((acc, s) => {
            const info = JSON.parse(s);
            const id = Id.fromString(info.id);
            const chainId = chainIDfromDID(DID.parseFromId(id));
            if (txDataChainId === chainId) {
                return acc;
            }
            const promise = this.resolveDidDocumentEip712MessageAndSignature(DID.parseFromId(Id.fromString(info.id)), didResolverUrl, {
                [type]: Hash.fromString(info[type])
            });
            return [...acc, promise];
        }, []);
    }
    static async prepareTxArgsSubmitV2(resolverUrl, txData, zkProofResponses) {
        if (txData.method_id.replace('0x', '') !== FunctionSignatures.SubmitZKPResponseV2) {
            throw new Error(`submit cross chain doesn't implement requested method id. Only '0x${FunctionSignatures.SubmitZKPResponseV2}' is supported.`);
        }
        const gistUpdates = [];
        const stateUpdates = [];
        const payload = [];
        const emptyBytes = '0x';
        for (const zkProof of zkProofResponses) {
            const { id: requestId, pub_signals: inputs } = zkProof;
            const proofCircuitId = zkProof.circuitId;
            if (!this._supportedCircuits.includes(proofCircuitId)) {
                throw new Error(`Circuit ${zkProof.circuitId} not supported by OnChainZKPVerifier`);
            }
            if (inputs.length === 0) {
                payload.push({
                    requestId: requestId,
                    zkProof: emptyBytes,
                    data: emptyBytes
                });
                continue;
            }
            const preparedZkpProof = prepareZkpProof(zkProof.proof);
            const zkProofEncoded = packZkpProof(inputs, preparedZkpProof.a, preparedZkpProof.b, preparedZkpProof.c);
            const stateInfo = this.getOnChainGistRootStatePubSignals(proofCircuitId, inputs);
            const chainId = txData.chain_id;
            const gistUpdateResolutions = this.getCrossChainResolvers(stateInfo.gists, chainId, 'gist', resolverUrl);
            const stateUpdateResolutions = this.getCrossChainResolvers(stateInfo.states, chainId, 'state', resolverUrl);
            if (gistUpdateResolutions.length > 0) {
                gistUpdates.push(...(await Promise.all(gistUpdateResolutions)));
            }
            if (stateUpdateResolutions.length > 0) {
                stateUpdates.push(...(await Promise.all(stateUpdateResolutions)));
            }
            const metadataArr = [];
            if (zkProof.vp) {
                for (const key in zkProof.vp.verifiableCredential.credentialSubject) {
                    if (key === '@type') {
                        continue;
                    }
                    const metadataValue = poseidon.hashBytes(byteEncoder.encode(JSON.stringify(zkProof.vp.verifiableCredential.credentialSubject[key])));
                    const bytesValue = byteEncoder.encode(metadataValue.toString());
                    metadataArr.push({
                        key,
                        value: bytesValue
                    });
                }
            }
            const metadata = metadataArr.length ? this.packMetadatas(metadataArr) : emptyBytes;
            payload.push({
                requestId: requestId,
                zkProof: zkProofEncoded,
                data: metadata
            });
        }
        const crossChainProofs = gistUpdates.length || stateUpdates.length
            ? this.packCrossChainProofs(gistUpdates, stateUpdates)
            : emptyBytes;
        return [payload, crossChainProofs];
    }
    async prepareTxArgsSubmitV2(txData, zkProofResponses) {
        if (!this._opts?.didResolverUrl) {
            throw new Error(`did resolver url required for crosschain verification`);
        }
        return OnChainZKPVerifier.prepareTxArgsSubmitV2(this._opts.didResolverUrl, txData, zkProofResponses);
    }
    static async _processProof(zkProof) {
        const requestID = zkProof.id;
        const inputs = zkProof.pub_signals;
        if (!this._supportedCircuits.includes(zkProof.circuitId)) {
            throw new Error(`Circuit ${zkProof.circuitId} not supported by OnChainZKPVerifier`);
        }
        const preparedZkpProof = prepareZkpProof(zkProof.proof);
        const zkProofEncoded = packZkpProof(inputs, preparedZkpProof.a, preparedZkpProof.b, preparedZkpProof.c);
        const metadataArr = [];
        if (zkProof.vp) {
            for (const key in zkProof.vp.verifiableCredential.credentialSubject) {
                if (key === '@type') {
                    continue;
                }
                const metadataValue = poseidon.hashBytes(byteEncoder.encode(JSON.stringify(zkProof.vp.verifiableCredential.credentialSubject[key])));
                const bytesValue = byteEncoder.encode(metadataValue.toString());
                metadataArr.push({
                    key,
                    value: bytesValue
                });
            }
        }
        const metadata = metadataArr.length ? this.packMetadatas(metadataArr) : '0x';
        return { requestID, zkProofEncoded, metadata };
    }
    static packCrossChainProofs(gistUpdateArr, stateUpdateArr) {
        const proofs = [];
        for (const globalStateUpdate of gistUpdateArr) {
            proofs.push({
                proofType: 'globalStateProof',
                proof: this.packGlobalStateMsg(globalStateUpdate)
            });
        }
        for (const stateUpdate of stateUpdateArr) {
            proofs.push({
                proofType: 'stateProof',
                proof: this.packIdentityStateMsg(stateUpdate)
            });
        }
        return new ethers.AbiCoder().encode(['tuple(' + 'string proofType,' + 'bytes proof' + ')[]'], [proofs]);
    }
    static packGlobalStateMsg(msg) {
        return new ethers.AbiCoder().encode([
            'tuple(' +
                'tuple(' +
                'uint256 timestamp,' +
                'bytes2 idType,' +
                'uint256 root,' +
                'uint256 replacedAtTimestamp' +
                ') globalStateMsg,' +
                'bytes signature,' +
                ')'
        ], [msg]);
    }
    static packIdentityStateMsg(msg) {
        return new ethers.AbiCoder().encode([
            'tuple(' +
                'tuple(' +
                'uint256 timestamp,' +
                'uint256 id,' +
                'uint256 state,' +
                'uint256 replacedAtTimestamp' +
                ') idStateMsg,' +
                'bytes signature,' +
                ')'
        ], [msg]);
    }
    static packMetadatas(metas) {
        return new ethers.AbiCoder().encode(['tuple(' + 'string key,' + 'bytes value' + ')[]'], [metas]);
    }
    static getOnChainGistRootStatePubSignals(onChainCircuitId, inputs) {
        const PubSignals = this._supportedCircuitsPubSignalsMap[onChainCircuitId];
        if (!PubSignals) {
            throw new Error(`Circuit ${onChainCircuitId} not supported by OnChainZKPVerifier`);
        }
        const atomicQueryPubSignals = new PubSignals();
        const encodedInputs = byteEncoder.encode(JSON.stringify(inputs));
        atomicQueryPubSignals.pubSignalsUnmarshal(encodedInputs);
        return atomicQueryPubSignals.getStatesInfo();
    }
    static async resolveDidDocumentEip712MessageAndSignature(did, resolverUrl, opts) {
        const didDoc = await resolveDidDocument(did, resolverUrl, {
            ...opts,
            signature: DIDDocumentSignature.EthereumEip712Signature2021
        });
        if (!didDoc.didResolutionMetadata.proof?.length) {
            throw new Error('No proof found in resolved DID document');
        }
        const message = didDoc.didResolutionMetadata.proof[0].eip712.message;
        const signature = didDoc.didResolutionMetadata.proof[0].proofValue;
        const isGistRequest = opts?.gist && !opts.state;
        if (isGistRequest) {
            return {
                globalStateMsg: {
                    timestamp: message.timestamp,
                    idType: message.idType,
                    root: message.root,
                    replacedAtTimestamp: message.replacedAtTimestamp
                },
                signature
            };
        }
        return {
            idStateMsg: {
                timestamp: message.timestamp,
                id: message.id,
                state: message.state,
                replacedAtTimestamp: message.replacedAtTimestamp
            },
            signature
        };
    }
}
/**
 * supported circuits
 */
OnChainZKPVerifier._supportedCircuits = [
    CircuitId.AuthV2,
    CircuitId.AtomicQueryMTPV2OnChain,
    CircuitId.AtomicQuerySigV2OnChain,
    CircuitId.AtomicQueryV3OnChain
];
OnChainZKPVerifier._supportedCircuitsPubSignalsMap = {
    [CircuitId.AtomicQueryMTPV2OnChain]: AtomicQueryMTPV2OnChainPubSignals,
    [CircuitId.AtomicQuerySigV2OnChain]: AtomicQuerySigV2OnChainPubSignals,
    [CircuitId.AtomicQueryV3OnChain]: AtomicQueryV3OnChainPubSignals,
    [CircuitId.AuthV2]: AuthV2PubSignals
};
